import java.util.*;

public class hello_finn {
    public static void main(String[] args) {
        Scanner sc = new Scanner(System.in);
        String line = sc.nextLine();
        System.out.printf("Hello %s\n", line);
    }
}