/**
 * Example solution with a name override.
 *
 * crifx!(author=Jane Doe)
 */

public class hello_world {
    public static void main(String[] args) {
        System.out.println("Hello world!");
    }
}